#!/bin/sh

#
# Copyright (c) 2008 Peter Holm
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: all.sh,v 1.2 2008/05/10 18:45:49 pho Exp $
#

# Run all the scripts in stress2/misc

[ `id -u ` -ne 0 ] && echo "Must be root!" && exit 1

# Start of list			Run	Known problems					Verified

# altbufferflushes.sh		Y
# alternativeFlushPath.sh	Y
# backingstore.sh		Y
# crossmp.sh			Y
# crossmp2.sh			N	panic: sx lock still held			20071101
# devfs.sh			Y
# devfs2.sh			Y							20070503
# fdescfs.sh			Y
# fuzz.sh			N							20080413
# inversion.sh			N	Problem not seen lately
# kevent.sh			Y	panic: KN_INFLUX set when not suppose to be	20080501
# lockf.sh			Y	Page fault in nfs_advlock			20080413
# mac.sh			Y
# md.sh				N	Waiting for fix					20071208
# mmap.sh.sh			N	Waiting for fix					20080222
# mount.sh			N	Known problem					20070505
# mount2.sh			Y
# msdos.sh			Y
# newfs.sh			Y
# newfs2.sh			Y
# newfs3.sh			N	panic: lockmgr: locking against myself		20070505
# nfs.sh			Y
# nullfs.sh			Y							Never
# quota1.sh			Y
# quota2.sh			Y
# quota3.sh			Y
# quota4.sh			N	Known backing store problem			20070703
# quota5.sh			Y
# quota6.sh			N	Known problem with snapshots and no disk space
# quota7.sh			Y							20070505
# quota8.sh			Y							20070505
# recursiveflushes.sh		Y
# revoke.sh			Y
# snap.sh			N	Waiting for snap3.sh fix
# snap2-1.sh			Y
# snap2.sh			Y
# snap3.sh			N	Reported as kostik033.html
# snap4.sh			Y
# snap5-1.sh			Y
# snap5.sh			Y
# snap6.sh			Y
# snap7.sh			N	Waiting for snap3.sh fix			20070508
# statfs.sh			Y
# umount.sh			Y
# umountf.sh			Y
# unionfs.sh			N 	Page fault					20070503
# unionfs2.sh			N	Reported as cons224				20070504
# unionfs3.sh			N	Page fault in vfs_statfs			20070504

# End of list

list=`sed -n '/^# Start of list/,/^# End of list/p' < $0 | awk '$3 ~ /Y/ {print $2}'`

# Shuffle the list
list=`perl -e 'print splice(@ARGV,rand(@ARGV),1), " " while @ARGV;' $list`

for i in $list; do
   echo "`date '+%Y%m%d %T'` $i" | tee /dev/tty >> /tmp/misc.log
   logger "Starting test $i"
   rm -f /tmp/.snap/pho* /var/.snap/pho*
   ./$i
done
